@@ -12,6 +12,7 @@
 #include "vcard.h"
 #include "vcard_emul.h"
 #include "card_7816.h"
+#include "common.h"
 
 
 /* Global Platform Card Manager applet AID */
@@ -563,7 +564,8 @@ vcard_make_response(vcard_7816_status_t status)
  * Add File card support here if you need it.
  */
 static VCardStatus
-vcard7816_file_system_process_apdu(VCard *card, VCardAPDU *apdu,
+vcard7816_file_system_process_apdu(G_GNUC_UNUSED VCard *card,
+                                   G_GNUC_UNUSED VCardAPDU *apdu,
                                    VCardResponse **response)
 {
     /* TODO: if we want to support a virtual file system card, we do it here.
@@ -635,6 +637,7 @@ vcard7816_vm_process_apdu(VCard *card, VCardAPDU *apdu,
                             VCARD7816_STATUS_ERROR_FUNCTION_NOT_SUPPORTED);
             break;
         }
+        g_debug("%s: Selecting file %s", __func__, hex_dump(apdu->a_body, apdu->a_Lc));
 
         /* side effect, deselect the current applet if no applet has been found
          */
@@ -660,8 +663,12 @@ vcard7816_vm_process_apdu(VCard *card, VCardAPDU *apdu,
                     sizeof(gp_response), apdu->a_Le, VCARD7816_STATUS_SUCCESS);
             } else {
                 unsigned char fci_template[] = {
-                    0x6F, 0x0B, 0x84, 0x07, 0xA0, 0x00, 0x00, 0x00,
-                    0x79, 0x03, 0x00, 0xA5, 0x00};
+                    0x6F, 0x0B, /* Outer lenght to be replaced later */
+                    0x84, 0x07, /* AID length to be replaced later */
+                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* AID */
+                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                    0xA5, 0x00}; /* The rest */
+                size_t fci_template_len = 6 + apdu->a_Lc;
                 /* with GSC-IS 2 applets, we do not need to return anything
                  * for select applet, but cards generally do, at least this
                  * FCI template stub:
@@ -672,10 +679,14 @@ vcard7816_vm_process_apdu(VCard *card, VCardAPDU *apdu,
                  *  A5 00 : Proprietary data
                  */
                 /* Insert the correct AID in the structure */
-                g_assert_cmpint(apdu->a_Lc, ==, 7);
+                g_assert_cmpint(fci_template_len, <=, sizeof(fci_template));
+                fci_template[1] = apdu->a_Lc + 4;
+                fci_template[3] = apdu->a_Lc;
                 memcpy(&fci_template[4], apdu->a_body, apdu->a_Lc);
+                fci_template[apdu->a_Lc + 4] = 0xA5;
+                fci_template[apdu->a_Lc + 5] = 0x00;
                 *response = vcard_response_new(card, fci_template,
-                    sizeof(fci_template), apdu->a_Le, VCARD7816_STATUS_SUCCESS);
+                    fci_template_len, apdu->a_Le, VCARD7816_STATUS_SUCCESS);
             }
         } else {
             /* the real CAC returns (SW1=0x6A, SW2=0x82) */
@@ -690,8 +701,13 @@ vcard7816_vm_process_apdu(VCard *card, VCardAPDU *apdu,
                             VCARD7816_STATUS_ERROR_WRONG_PARAMETERS);
         } else {
             if (apdu->a_Lc == 0) {
-                /* handle pin count if possible */
-                count = vcard_emul_get_login_count(card);
+                /* If we are already logged in, we should succeed just now */
+                if (vcard_emul_is_logged_in(card)) {
+                    *response = vcard_make_response(VCARD7816_STATUS_SUCCESS);
+                    break;
+                }
+                /* handle pin count if possible (not possible now) */
+                count = vcard_get_login_count(card);
                 if (count < 0) {
                     *response = vcard_make_response(
                                     VCARD7816_STATUS_ERROR_DATA_NOT_FOUND);
