@@ -32,6 +32,9 @@ struct VCardStruct {
     VCardEmul *vcard_private;
     VCardEmulFree vcard_private_free;
     VCardGetAtr vcard_get_atr;
+    unsigned int compat;
+    unsigned char serial[32]; /* SHA256 of the first certificate */
+    int serial_len;
 };
 
 VCardBufferResponse *
@@ -141,6 +144,8 @@ vcard_new(VCardEmul *private, VCardEmulFree private_free)
 {
     VCard *new_card;
 
+    g_debug("%s: called", __func__);
+
     new_card = g_new0(VCard, 1);
     new_card->type = VCARD_VM;
     new_card->vcard_private = private;
@@ -204,6 +209,8 @@ vcard_set_atr_func(VCard *card, VCardGetAtr get_atr)
 VCardStatus
 vcard_add_applet(VCard *card, VCardApplet *applet)
 {
+    g_debug("%s: called", __func__);
+
     applet->next = card->applet_list;
     card->applet_list = applet;
     /* if our card-type is direct, always call the applet */
@@ -319,3 +326,40 @@ vcard_get_private(VCard *vcard)
     return vcard->vcard_private;
 }
 
+/* Get remaining login count for the current card */
+int
+vcard_get_login_count(VCard *card)
+{
+    int rv = vcard_emul_get_login_count(card);
+
+    /* Windows drivers are not very happy with the answer we can give here
+     * so lets assume the card still have all the attempts unlocked here */
+    if (rv == -1 && card->compat & VCARD_COMPAT_WINDOWS) {
+        return 3;
+    }
+    return rv;
+}
+
+/* Set compat bits for the given cards. See VCARD_COMPAT_* options */
+void
+vcard_set_compat(VCard *card, unsigned int set)
+{
+    card->compat |= set;
+}
+
+/* Set card serial number */
+void
+vcard_set_serial(VCard *card, unsigned char *serial, size_t len)
+{
+    len = MIN(sizeof(card->serial), len);
+    memcpy(card->serial, serial, len);
+    card->serial_len = len;
+}
+
+unsigned char *
+vcard_get_serial(VCard *card, int *len)
+{
+    if (len != NULL)
+        *len = card->serial_len;
+    return card->serial;
+}
