@@ -21,10 +21,13 @@
 #include <nss.h>
 #include <pk11pub.h>
 #include <cert.h>
-#include <key.h>
+#include <keyhi.h>
 #include <secmod.h>
 #include <prthread.h>
 #include <secerr.h>
+#include <secoid.h>
+#include <secmodt.h>
+#include <sechash.h>
 
 #include "glib-compat.h"
 
@@ -399,7 +402,7 @@ cleanup:
 /* return the number of login attempts still possible on the card. if unknown,
  * return -1 */
 int
-vcard_emul_get_login_count(VCard *card)
+vcard_emul_get_login_count(G_GNUC_UNUSED VCard *card)
 {
     return -1;
 }
@@ -446,6 +449,32 @@ vcard_emul_login(VCard *card, unsigned char *pin, int pin_len)
     return VCARD7816_STATUS_ERROR_CONDITION_NOT_SATISFIED;
 }
 
+int
+vcard_emul_is_logged_in(VCard *card)
+{
+    PK11SlotInfo *slot;
+
+    if (!nss_emul_init) {
+        return VCARD7816_STATUS_ERROR_CONDITION_NOT_SATISFIED;
+    }
+
+    slot = vcard_emul_card_get_slot(card);
+     /* We depend on the PKCS #11 module internal login state here because we
+      * create a separate process to handle each guest instance. If we needed
+      * to handle multiple guests from one process, then we would need to keep
+      * a lot of extra state in our card structure
+      */
+
+    /* If we do not need log in, we present the token as "logged in" */
+    if (PK11_NeedLogin(slot) == PR_FALSE) {
+        return 1;
+    }
+
+    /* For the tokens that require login, delegate to NSS to figure out the
+     * login status */
+    return !!PK11_IsLoggedIn(slot, NULL);
+}
+
 void
 vcard_emul_logout(VCard *card)
 {
@@ -462,7 +491,7 @@ vcard_emul_logout(VCard *card)
 }
 
 void
-vcard_emul_reset(VCard *card, VCardPower power)
+vcard_emul_reset(VCard *card, G_GNUC_UNUSED VCardPower power)
 {
     /*
      * if we reset the card (either power on or power off), we lose our login
@@ -581,7 +610,7 @@ static unsigned char *nss_atr;
 static int nss_atr_len;
 
 void
-vcard_emul_get_atr(VCard *card, unsigned char *atr, int *atr_len)
+vcard_emul_get_atr(G_GNUC_UNUSED VCard *card, unsigned char *atr, int *atr_len)
 {
     int len;
     assert(atr != NULL);
@@ -594,6 +623,30 @@ vcard_emul_get_atr(VCard *card, unsigned char *atr, int *atr_len)
     *atr_len = len;
 }
 
+static SECStatus
+vcard_emul_create_serial(VCard *card, unsigned char *data, int len)
+{
+    HASH_HashType hashType;
+    HASHContext *hashContext = NULL;
+    unsigned char digest[32];
+    unsigned int digestLen = 0;
+
+    hashType = HASH_GetHashTypeByOidTag(SEC_OID_SHA256);
+    hashContext = HASH_Create(hashType);
+    if (hashContext == NULL) {
+        return SECFailure;
+    }
+
+    HASH_Begin(hashContext);
+    HASH_Update(hashContext, data, len);
+    HASH_End(hashContext, digest, &digestLen, 32);
+    HASH_Destroy(hashContext);
+
+    vcard_set_serial(card, digest, (size_t) digestLen);
+
+    return SECSuccess;
+}
+
 /*
  * create a new card from certs and keys
  */
@@ -608,6 +661,8 @@ vcard_emul_make_card(VReader *reader,
     VCardEmulType type;
     const char *params;
 
+    g_debug("%s: called", __func__);
+
     type = vcard_emul_get_type(reader);
 
     /* ignore the inserted card */
@@ -631,6 +686,11 @@ vcard_emul_make_card(VReader *reader,
         vcard_emul_delete_card(vcard_emul);
         return NULL;
     }
+
+    if (cert_count > 0) {
+        vcard_emul_create_serial(vcard, certs[0], cert_len[0]);
+    }
+
     vcard_init(reader, vcard, type, params, certs, cert_len, keys, cert_count);
     return vcard;
 }
@@ -654,6 +714,8 @@ vcard_emul_mirror_card(VReader *vreader)
     PK11SlotInfo *slot;
     VCard *card;
 
+    g_debug("%s: called", __func__);
+
     slot = vcard_emul_reader_get_slot(vreader);
     if (slot == NULL) {
         return NULL;
@@ -682,6 +744,7 @@ vcard_emul_mirror_card(VReader *vreader)
         CERTCertificate *cert;
         SECStatus rv;
 
+        g_debug("%s: Found certificate", __func__);
         rv = PK11_ReadRawAttribute(PK11_TypeGeneric, thisObj,
                                    CKA_VALUE, &derCert);
         if (rv != SECSuccess) {
@@ -788,7 +851,7 @@ vcard_emul_event_thread(void *arg)
 
 /* if the card is inserted when we start up, make sure our state is correct */
 static void
-vcard_emul_init_series(VReader *vreader, VCard *vcard)
+vcard_emul_init_series(VReader *vreader, G_GNUC_UNUSED VCard *vcard)
 {
     VReaderEmul *vreader_emul = vreader_get_private(vreader);
     PK11SlotInfo *slot = vreader_emul->slot;
@@ -829,7 +892,7 @@ static const VCardEmulOptions default_options = {
  *  NULL.
  */
 static char *
-vcard_emul_get_password(PK11SlotInfo *slot, PRBool retries, void *pw_arg)
+vcard_emul_get_password(G_GNUC_UNUSED PK11SlotInfo *slot, PRBool retries, void *pw_arg)
 {
     /* if it didn't work the first time, don't keep trying */
     if (retries) {
@@ -905,6 +968,8 @@ vcard_emul_init(const VCardEmulOptions *options)
     SECMODModuleList *mlp;
     int i;
 
+    g_debug("%s: called", __func__);
+
     if (vcard_emul_init_called) {
         return VCARD_EMUL_INIT_ALREADY_INITED;
     }
@@ -929,6 +994,7 @@ vcard_emul_init(const VCardEmulOptions *options)
             return VCARD_EMUL_FAIL;
         }
 
+        g_debug("%s: returning with passthrough initialized", __func__);
         return VCARD_EMUL_OK;
     }
 #endif
@@ -1018,6 +1084,7 @@ vcard_emul_init(const VCardEmulOptions *options)
     /* if we aren't suppose to use hw, skip looking up hardware tokens */
     if (!options->use_hw) {
         nss_emul_init = has_readers;
+        g_debug("%s: returning: Not using HW", __func__);
         return has_readers ? VCARD_EMUL_OK : VCARD_EMUL_FAIL;
     }
 
@@ -1040,6 +1107,7 @@ vcard_emul_init(const VCardEmulOptions *options)
             continue;
         }
 
+        g_debug("%s: Listing modules, trying %s", __func__, module->commonName);
         for (i = 0; i < module->slotCount; i++) {
             PK11SlotInfo *slot = module->slots[i];
 
@@ -1063,12 +1131,16 @@ vcard_emul_init(const VCardEmulOptions *options)
             vreader = vreader_new(PK11_GetSlotName(slot), vreader_emul,
                                   vreader_emul_delete);
             vreader_add_reader(vreader);
+            g_debug("%s: Added reader from slot %s", __func__,
+                    PK11_GetSlotName(slot));
 
             if (PK11_IsPresent(slot)) {
                 VCard *vcard;
                 vcard = vcard_emul_mirror_card(vreader);
                 vreader_insert_card(vreader, vcard);
                 vcard_emul_init_series(vreader, vcard);
+                g_debug("%s: Added card to the reader %s", __func__,
+                        vreader_get_name(vreader));
                 vcard_free(vcard);
             }
         }
