@@ -14,23 +14,28 @@
 #include "vcard_emul_type.h"
 #include "cac.h"
 #include "gp.h"
+#include "msft.h"
 #include "glib-compat.h"
 
 VCardStatus vcard_init(VReader *vreader, VCard *vcard,
-                       VCardEmulType type, const char *params,
+                       VCardEmulType type, G_GNUC_UNUSED const char *params,
                        unsigned char *const *cert, int cert_len[],
                        VCardKey *key[], int cert_count)
 {
     int rv;
 
+    g_debug("%s: called", __func__);
+
     switch (type) {
     case VCARD_EMUL_NONE:
         break;
     case VCARD_EMUL_CAC:
-        rv = cac_card_init(vreader, vcard, params,
-            cert, cert_len, key,  cert_count);
+        rv = cac_card_init(vreader, vcard,
+            cert, cert_len, key, cert_count);
         if (rv == VCARD_DONE)
             rv = gp_card_init(vreader, vcard);
+        if (rv == VCARD_DONE)
+            rv = msft_card_init(vreader, vcard);
         return rv;
     /* add new ones here */
     case VCARD_EMUL_PASSTHRU:
@@ -41,7 +46,7 @@ VCardStatus vcard_init(VReader *vreader, VCard *vcard,
     return VCARD_FAIL;
 }
 
-VCardEmulType vcard_emul_type_select(VReader *vreader)
+VCardEmulType vcard_emul_type_select(G_GNUC_UNUSED VReader *vreader)
 {
     /* return the default */
     return VCARD_EMUL_CAC;
