@@ -127,6 +127,10 @@ static void do_login(VReader *reader)
         //0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37, 0x37
         0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
     };
+    uint8_t login_check[] = {
+        /* VERIFY   [p1,p2=0 ]  [Lc] */
+        0x00, 0x20, 0x00, 0x00, 0x00
+    };
     int login_len, pin_len;
 
     g_assert_nonnull(reader);
@@ -143,6 +147,14 @@ static void do_login(VReader *reader)
     g_assert_cmpint(status, ==, VREADER_OK);
     g_assert_cmphex(pbRecvBuffer[0], ==, VCARD7816_SW1_SUCCESS);
     g_assert_cmphex(pbRecvBuffer[1], ==, 0x00);
+
+    /* Check the login status now */
+    status = vreader_xfr_bytes(reader,
+                               login_check, sizeof(login_check),
+                               pbRecvBuffer, &dwRecvLength);
+    g_assert_cmpint(status, ==, VREADER_OK);
+    g_assert_cmphex(pbRecvBuffer[0], ==, VCARD7816_SW1_SUCCESS);
+    g_assert_cmphex(pbRecvBuffer[1], ==, 0x00);
 }
 
 static void test_passthrough_applets(void)
@@ -355,6 +367,8 @@ int main(int argc, char *argv[])
     g_test_add_func("/hw-tests/list", test_list);
     g_test_add_func("/hw-tests/passthrough-applet", test_passthrough_applets);
     g_test_add_func("/hw-tests/check-login-count", check_login_count);
+    g_test_add_func("/hw-tests/msft-applet", check_login_count);
+    g_test_add_func("/hw-tests/gp-applet", test_gp_applet);
     g_test_add_func("/hw-tests/login", test_login);
     g_test_add_func("/hw-tests/sign", test_sign);
     g_test_add_func("/hw-tests/sign-bad-data", test_sign_bad_data_x509);
