@@ -475,6 +475,10 @@ static void do_login(VReader *reader)
         /* VERIFY   [p1,p2=0 ]  [Lc]  [empty pin padded to 6 chars     ] */
         0x00, 0x20, 0x00, 0x00, 0x06, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
     };
+    uint8_t login_check[] = {
+        /* VERIFY   [p1,p2=0 ]  [Lc] */
+        0x00, 0x20, 0x00, 0x00, 0x00
+    };
     g_assert_nonnull(reader);
     status = vreader_xfr_bytes(reader,
                                login, sizeof(login),
@@ -482,6 +486,14 @@ static void do_login(VReader *reader)
     g_assert_cmpint(status, ==, VREADER_OK);
     g_assert_cmphex(pbRecvBuffer[0], ==, VCARD7816_SW1_SUCCESS);
     g_assert_cmphex(pbRecvBuffer[1], ==, 0x00);
+
+    /* Check the login status now */
+    status = vreader_xfr_bytes(reader,
+                               login_check, sizeof(login_check),
+                               pbRecvBuffer, &dwRecvLength);
+    g_assert_cmpint(status, ==, VREADER_OK);
+    g_assert_cmphex(pbRecvBuffer[0], ==, VCARD7816_SW1_SUCCESS);
+    g_assert_cmphex(pbRecvBuffer[1], ==, 0x00);
 }
 
 static void test_cac_pki(void)
@@ -625,54 +637,6 @@ static void test_select_coid(void)
     vreader_free(reader); /* get by id ref */
 }
 
-static void test_gp_applet(void)
-{
-    int dwRecvLength = APDUBufSize;
-    VReaderStatus status;
-    uint8_t pbRecvBuffer[APDUBufSize];
-    uint8_t gp_aid[] = {
-        0xA0, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00
-    };
-    uint8_t getresp[] = {
-        /* Get Response (max we can get) */
-        0x00, 0xc0, 0x00, 0x00, 0x00
-    };
-    uint8_t getdata[] = {
-        /* Get Response (max we can get) */
-        0x00, 0xca, 0x9f, 0x7f, 0x00
-    };
-    VReader *reader = vreader_get_reader_by_id(0);
-
-    /* select GP and wait for the response bytes */
-    select_aid_response(reader, gp_aid, sizeof(gp_aid), 0x1b);
-
-    /* read the response from the card */
-    dwRecvLength = APDUBufSize;
-    status = vreader_xfr_bytes(reader,
-                               getresp, sizeof(getresp),
-                               pbRecvBuffer, &dwRecvLength);
-    g_assert_cmpint(status, ==, VREADER_OK);
-    g_assert_cmpint(dwRecvLength, >, 2);
-    g_assert_cmphex(pbRecvBuffer[dwRecvLength-2], ==, VCARD7816_SW1_SUCCESS);
-    g_assert_cmphex(pbRecvBuffer[dwRecvLength-1], ==, 0x00);
-
-    /* We made sure the selection of other applets does not return anything
-     * in select_aid()
-     */
-
-    /* ask the applet for some data */
-    dwRecvLength = APDUBufSize;
-    status = vreader_xfr_bytes(reader,
-                               getdata, sizeof(getdata),
-                               pbRecvBuffer, &dwRecvLength);
-    g_assert_cmpint(status, ==, VREADER_OK);
-    g_assert_cmpint(dwRecvLength, ==, 0x2F);
-    g_assert_cmpint(pbRecvBuffer[dwRecvLength-2], ==, VCARD7816_SW1_SUCCESS);
-    g_assert_cmpint(pbRecvBuffer[dwRecvLength-1], ==, 0x00);
-
-    vreader_free(reader); /* get by id ref */
-}
-
 static void test_invalid_properties(void)
 {
     VReader *reader = vreader_get_reader_by_id(0);
@@ -1086,6 +1050,25 @@ static void test_passthrough_applet(void)
     vreader_free(reader); /* get by id ref */
 }
 
+#define MAX_ATR_LEN 100
+#define CAC_ATR "\x3B\x7A\x18\x00\x00\x73\x66\x74\x65\x20\x63\x64\x31\x34\x34"
+#define CAC_ATR_LEN (sizeof(CAC_ATR) - 1)
+static void test_atr(void)
+{
+    VReader *reader = vreader_get_reader_by_id(0);
+    unsigned char *atr = g_malloc0(MAX_ATR_LEN);
+    int atr_len = MAX_ATR_LEN;
+
+    /* Cycle off and on to get the ATR from the card */
+    vreader_power_off(reader);
+    vreader_power_on(reader, atr, &atr_len);
+
+    g_assert_cmpmem(atr, atr_len, CAC_ATR, CAC_ATR_LEN);
+
+    vreader_free(reader); /* get by id ref */
+    g_free(atr);
+}
+
 static void libcacard_finalize(void)
 {
     VReader *reader = vreader_get_reader_by_id(0);
@@ -1125,14 +1108,16 @@ int main(int argc, char *argv[])
     g_test_add_func("/libcacard/sign", test_sign);
     g_test_add_func("/libcacard/empty-applets", test_empty_applets);
     g_test_add_func("/libcacard/gp-applet", test_gp_applet);
+    g_test_add_func("/libcacard/msft-applet", test_msft_applet);
     g_test_add_func("/libcacard/invalid-properties-apdu", test_invalid_properties);
     g_test_add_func("/libcacard/invalid-select-apdu", test_invalid_select);
     g_test_add_func("/libcacard/invalid-instruction", test_invalid_instruction);
     g_test_add_func("/libcacard/invalid-read-buffer", test_invalid_read_buffer);
     g_test_add_func("/libcacard/invalid-acr", test_invalid_acr);
+    g_test_add_func("/libcacard/get-atr", test_atr);
     /* Even without the card, the passthrough applets are present */
     g_test_add_func("/libcacard/passthrough-applet", test_passthrough_applet);
-    /* TODO: Card/reader resets + ATR tests */
+    /* TODO: Card/reader resets */
     g_test_add_func("/libcacard/remove", test_remove);
 
     ret = g_test_run();
